package ui.controller;

import java.util.ArrayList;
import java.util.LinkedList;

import com.sun.xml.internal.bind.v2.schemagen.xmlschema.List;

import Interfaces.CategoryListener;
import Interfaces.ComparableObject;
import ui.model.*;
import ui.view.*;

public class CategoryController {

	private IdCounter ID;
	private Model M;

	public CategoryController(Model model, IdCounter id) {
		this.ID = id;
		this.M = model;
		initCategories();
	}

	/**
	 * init default category and objects
	 */
	public void initCategories() {
		Category energy = new Category("Energy");
		Category building = new Category("Building");
		Category component = new Category("Component");

		HolonObject powerp = new HolonObject("Power Plant");
		HolonObject house = new HolonObject("House");
		HolonTransformer transformer = new HolonTransformer("Transformer");
		HolonSwitch sw = new HolonSwitch("Switch");

		addObject(energy, powerp);
		addObject(building, house);
		addObject(component, transformer);
		addObject(component, sw);

		addCategory(energy);
		addCategory(building);
		addCategory(component);

	}

	/**
	 * Adds Category into Model
	 * if a Category with the same name already exists
	 * Add Category_+1
	 * 
	 * @param toAdd
	 *            neue Kategorie
	 */
	public void addCategory(Category toAdd) {
		int number = 0;
		String name = toAdd.getName();
		while(containsInList( M.getCategories(),toAdd)){
			number++;
			toAdd.setName(name + "_" + number);
		};
		M.getCategories().add(toAdd);
		notifyCatListeners();
	}
	
	public void deleteCategory(int idx) {
		M.getCategories().remove(idx);
	}

	/**
	 * Adds New Category into Model
	 * 
	 * @param name
	 *            Bezeichnung der neuen Kategorie
	 */
	public void addNewCategory(String name) {

		addCategory(new Category(name));
	}

	/**
	 * Add Object into a Category
	 * 
	 * @param cat
	 *            Category
	 * @param obj
	 *            Object
	 */
	public void addObject(Category cat, CpsObject obj) {
		cat.getObjects().add(obj);
		notifyCatListeners();
	}

	/**
	 * Add new Holon Object
	 * 
	 * @param cat
	 *            Category
	 * @param obj
	 *            New Object Name
	 */
	public void addNewHolonObject(Category cat, String obj) {
		addObject(cat, new HolonObject(obj));
	}

	/**
	 * Add new Holon Transformer
	 * 
	 * @param cat
	 *            Category
	 * @param obj
	 *            New Object Name
	 */
	public void addNewHolonTransformer(Category cat, String obj) {
		addObject(cat, new HolonTransformer(obj));
	}

	/**
	 * Add new Holon Switch
	 * 
	 * @param cat
	 *            Category
	 * @param obj
	 *            New Object Name
	 */
	public void addNewHolonSwitch(Category cat, String obj) {
		addObject(cat, new HolonSwitch(obj));
	}

	/**
	 * 
	 */
	public void notifyCatListeners() {
		for (CategoryListener l : M.getCategoryListeners()) {
			l.onChange(M.getCategories());
		}
	}

	/**
	 * 
	 * @param catLis
	 */
	public void addCatListener(CategoryListener catLis) {
		M.getCategoryListeners().add(catLis);
	}

	/**
	 * search for category
	 * @param name
	 * @return
	 */
	public Category searchNode(String name) {
		Category query = null;

		for (Category cat : M.getCategories()) {
			if (cat.getName().equals(name)) {
				query = cat;
				break;
			}
		}
		return query;
	}

	public boolean containsInList(ArrayList<? extends ComparableObject> arrayList, ComparableObject toSearch){
		for(ComparableObject obj : arrayList){
			if(obj.getCompareName().equals(toSearch.getCompareName())){
				return true;
			}
		}
		return false;
	}
	

}
